function [effDimWL, DimWL, tstats, is_surface] = effdim(slip, params, doplot, override)
% [effDimWL, DimWL, tstats, is_surface] = effdim(slip, params, doplot, override)
% calculate the effective source dimensions for a given slip distribution.
% 
% INPUTS
% S:            2-D Slip Array
% params:       an structure array containing the following
%               - dip:      dip (degrees) of the fault
%               - htop:     top-depth of the fault
%               - samp:     spatial sampling   
%               - edgefact: (optional); this is same as "override" to define
%               edge artifacts in model, if present.
% doplot:       The options for plotting the figure. Default value is 0. 
%                   0  - do not plot
%                   1  - plot it on MATLAB fig file. 
%                   2  - same as option 1.
%                   3  - same as option 1, but only the slip distribution.
% override:     (optional) An integer array in form of [L, R, T, B], each
%               a value coresponds to number of subfualts to be trimmed 
%               on left, right, top and bottom. Providing this input argument 
%               overrides the algorithm by trimming the model according to the
%               provided number of subfualts to be trimmed (also see, mtrim.m)
%            
%
% NOTE
% We extend the approach the autocorrelation width difinition approach of Mai and
% Beroza (2000) 
%
% CITE
% Thingbaijam K.K.S., and P.M. Mai. Evidence for truncated exponential probability
%       distribution of earthquake slip, Bulletin of the Seismological Society of America,
%       under review, submitted October, 2015.
% Mai, P. M., and G.C. Beroza. Source scaling properties from finite-fault-rupture 
%       models. Bulletin of the Seismological Society of America, 90(3), 604-615.
%
% AUTHOR
% 2014, Thingbaijam, K.K.S. (thingbaijam@gmail.com)
%
% SEE 
% readme.pdf for a disclaimer. 
% http://www.equake-rc.info/cers-software/ 


% extract the source parameters
samp    = params.samp;
htop    = params.htop;
dip     = params.dip;

% check whether model have edgefact
if isfield(params, 'edgefact')
    edgefact    = params.edgefact;
    isedgefact  = 1;
    if edgefact == 0;
        isedgefact = 0;
    end
else
    isedgefact = 0;
end
   
% we do not want negative slip
slip = abs(slip);

if nargin < 3
    doplot =0;
end


S = slip;
% do we have surface rupture here?
is_surface = is_surface_rupture(S, samp(1), htop, dip);

acDimWL = autocorrwidth(S, samp);

if nargin >3 % a subjective override 
    tstats      = override;
    nznx        = size(S); NZ = nznx (1); NX = nznx(2);
    DimWL(1)    = NZ*samp(1);
    DimWL(2)    = NX*samp(2);
    effDimWL(1) = (NZ-(tstats(3)+tstats(4)))*samp(1);
    effDimWL(2) = (NX-(tstats(1)+tstats(2)))*samp(2);
else
    slip_asperity = max(S(:))/3.0;
    [effDimWL, tstats] = fitacDimWL(S,samp, acDimWL,slip_asperity,is_surface);

    if isedgefact==1 
        if edgefact ==1
            edgefact = [1, 1, 1, 1]; % L R T B 
        end    
        % check whether the edgefact has been trimmed 
        for fk=1:4
            if tstats(fk) < edgefact(fk)
                tstats(fk) = edgefact(fk);
            end
        end
        % adjust effdimWL for the change in tstats
        nznx = size(S);
        NZ = nznx(1); NX = nznx(2);
        effDimWL(1) = (NZ-(tstats(3)+tstats(4)))*samp(1);
        effDimWL(2) = (NX-(tstats(1)+tstats(2)))*samp(2);
    end
end
%

NzNx = size(slip);
DimWL = [NzNx(1)*samp(1), NzNx(2)*samp(2)];

if doplot>0
    % autocorrelation model 
    [autoDimWL, autostats] = fitacDimWL(S,samp, acDimWL,99999,0);
     % Borderlines for autocorr model
    axstart = autostats(1);
    if axstart>0
        axstart = axstart*samp(2);
    end
    axend = axstart+autoDimWL(2);
    azstart = autostats(3);
    if azstart>0
        azstart = azstart*samp(1);
    end
    azend = azstart+autoDimWL(1);
       
    % ---------
    SS = slip./100; 
    nznx = size(SS);
    
    % Borderlines for effective model
    xstart = tstats(1);
    if xstart>0
        xstart = xstart*samp(2);
    end
    xend = xstart+effDimWL(2);
    zstart = tstats(3);
    
    if zstart>0
        zstart = zstart*samp(1);
    end
    zend = zstart+effDimWL(1);
    
    %
    zzdir = 1:nznx(1);
    zzdir = zzdir.*samp(1) - samp(1)/2.0;
    xxdir = 1:nznx(2);
    xxdir = xxdir.*samp(2) - samp(2)/2.0;
    
    if doplot ==3
        imagesc(xxdir,zzdir,SS);    
        % load('slipcolor.mat');
        % colormap(slipcolor);
        colorbar('vert');   
        hold on;
    
        px = [xstart, xend, xend, xstart, xstart];
        py = [zstart, zstart, zend, zend, zstart];
        plot(px,py, 'k--', 'linewidth', 5); hold on;
    
        apx = [axstart, axend, axend, axstart, axstart];
        apy = [azstart, azstart, azend, azend, azstart];
        plot(apx,apy, 'k:', 'linewidth', 3); 
    
        set(gca, 'fontsize', 14);
         
        xlabel('Along strike (km)','fontsize', 14);
    
        set(gca,'YDir','Reverse')
        ylabel('Down dip (km)','fontsize', 14);  
        
        
    else
        subplot(3,3,[2,3,5,6])
        if doplot ==1
           imagesc(xxdir,zzdir,SS);    
        elseif doplot==2
           imagesc(xxdir,zzdir,SS); 
        end
        %load('slipcolor.mat');
        %colormap(slipcolor);
        colorbar('vert');
        title('Original model','fontsize', 13);
        hold on;
    
        px = [xstart, xend, xend, xstart, xstart];
        py = [zstart, zstart, zend, zend, zstart];
        plot(px,py, 'k--', 'linewidth', 5); hold on;
    
        apx = [axstart, axend, axend, axstart, axstart];
        apy = [azstart, azstart, azend, azend, azstart];
        plot(apx,apy, 'k:', 'linewidth', 3); 
    
        set(gca, 'fontsize', 14);
    
    
        bw = 0.3;
     
        S = S./100;
        % along strike ----------
        subplot(3,3,[8,9])
        Sc = max(S, [], 1);
        plot(xxdir, Sc, 'r-'); hold on;

        % plot two lines
        xxx = xstart:0.1:xend;
        yyy = ones(size(xxx)).*max(Sc)/2.0;
        plot(xxx, yyy, 'k-', 'linewidth',3);
        plot([xxx(1), xxx(1)],[yyy(1)-bw, yyy(1)+bw], 'k-', 'linewidth',3);
        plot([xxx(end), xxx(end)], [yyy(end)-bw, yyy(end)+bw],'k-', 'linewidth',3);
    
    
        axxx = axstart:0.1:axend;
        ayyy = ones(size(axxx)).*max(Sc)/1.5;
        plot(axxx, ayyy, 'k--', 'linewidth',2);
        plot([axxx(1), axxx(1)], [ayyy(1)-bw, ayyy(1)+bw], 'k-', 'linewidth',3);
        plot([axxx(end), axxx(end)],[ayyy(end)-bw, ayyy(end)+bw], 'k-', 'linewidth',3);
    
    
    
        %axis([0 max(xxdir) min(Sc) max(Sc)]);
        xlabel('Along strike (km)','fontsize', 14);
        ylabel('U_C_X (m)','fontsize', 14);
    
    
        % along dip
        subplot(3,3,[1,4])
        Sr = max(S, [], 2);
        plot(Sr, zzdir, 'r-'); hold on;
        % plot two lines
        xxx = zstart:0.1:zend;
        yyy = ones(size(xxx)).*max(Sr)/2.0;
        plot(yyy, xxx, 'k-', 'linewidth',3);
        plot([yyy(1)-bw, yyy(1)+bw], [xxx(1), xxx(1)], 'k-', 'linewidth',3);
        plot([yyy(end)-bw, yyy(end)+bw], [xxx(end), xxx(end)], 'k-', 'linewidth',3);
    
        axxx = azstart:0.1:azend;
        ayyy = ones(size(axxx)).*max(Sr)/1.5;
        plot(ayyy, axxx, 'k--', 'linewidth',2);
        plot([ayyy(1)-bw, ayyy(1)+bw], [axxx(1), axxx(1)], 'k-', 'linewidth',3);
        plot([ayyy(end)-bw, ayyy(end)+bw], [axxx(end), axxx(end)], 'k-', 'linewidth',3);
    
        set(gca,'YDir','Reverse')
        ylabel('Down dip (km)','fontsize', 14);
        xlabel('U_C_Z (m)','fontsize', 14);
    end
    
end



function [effDimWL, tstats] = fitacDimWL(S,samp, acDimWL,slip_asperity,srup)
% [effDimWL, tstats] = fitacDimWL(S,samp, acDimWL,slip_asperity,srup)
% returns the trim statistics by fitting the autocorrelation dimension of the slip distribution.
% For definition of autocorrelation width, see Mai and Beroza (2000)
%
% 2014, Thingbaijam K.K.S. 


% width/rows
Sz = max(S, [], 2);
[W, TB]  = fitautocorrdim(Sz, samp(1), acDimWL(1),slip_asperity, srup);

% length/colums
Sx = max(S, [], 1);

[L, LR] = fitautocorrdim(Sx, samp(2), acDimWL(2),slip_asperity);

% tstats = [L, R, T, B]
effDimWL = [W, L];
tstats = [LR(1), LR(2), TB(1), TB(2)];


function [D, tstats] = fitautocorrdim(Sr,dr,autodim, slip_asperity, surface_rupture)
% D = fitautocorrdim(Sr,dr,autodim)
%
% Finds the largest dimension that fits the autocorrelation width (dimension)
% iteratively. For definition of autocorrelation width, see Mai and Beroza
% (2000). Large slip asperity is also checked, not to discard/trim those
% regions. Also, if surface rupture is encountered, trimming is not done
% for the top of the fault. 
% 
% 2014, Thingbaijam K.K.S

if nargin<5
    surface_rupture = 0;
end

% just to be sure
Sr = Sr(:)'; oNfs = length(Sr);
St = Sr;

inc = 0.001; kcut = 0.001;
not_asperity_top = 1; not_asperity_bottom=1;
L = 0; R = 0;

while 1    
    % left/top
    if not_asperity_top==1
        if Sr(1) >= slip_asperity
            if L>0
                L = L-1;
            end
            not_asperity_top = 0;
        else
            if Sr(1) <= kcut    
                Sr(1) = [];
                L = L+1;
            end
       end
    end

    % right/bottom
    if not_asperity_bottom ==1   
        if Sr(end) >= slip_asperity
             if R>0
                 R=R-1;
             end
             not_asperity_bottom =0;
        else
             if Sr(end) <= kcut    
                Sr(end) = [];
                R=R+1;
            end
        end
    end
    
    if (not_asperity_top == 0) && (not_asperity_bottom==0)
        break;
    end
    
    newdim = length(Sr)*dr;
    if newdim-autodim <= 2*dr
        break;
    else
        kcut = kcut+inc;
    end
    
end

% check for asperity again!!
while 1
    if L > 0
        if St(L+1)>= slip_asperity
            L = L-1;
        else
            break;
        end
    else
        break;
    end
end

while 1
    if R > 0
        if St(end-R)>= slip_asperity
            R = R-1;
        else
            break;
        end
    else
        break;
    end
end

% for possible zeros band
while 1
    if St(L+1) <= 0
        L = L+1;
    else 
        break;
    end
end

% for possible zeros band
while 1
    if St(end-R) <= 0
        R = R+1;
    else 
        break;
    end
end

if surface_rupture
    L = 0;
end

tstats = [L,R];

D = (oNfs-(L+R))*dr;




function tf = is_surface_rupture(S, dz, htop, dip)
% tf = is_surface_rupture(S, dz, htop, dip)
% Self-explaining name of the function
% 
% Also see, Kagawa et al. (2004) for a similar definition. 
%
% 2014, Thingbaijam K.K.S.

% consider fixed 'sdepth' km depth from the top
sdepth = 5;  % km

tf = 0;
if htop >0.0
    return
end

if sum(S(1,:))< 0.0001
    return
end

significant_length  = sdepth/sind(dip);
nsiglen             = significant_length/dz;
nsiglen             = ceil(nsiglen);

% check for signifcant slip 
slip_asperity       = max(S(:))/3.0;
for i=1:nsiglen
    if max(S(i,:)) >= slip_asperity
        tf = 1; break;
    end
end


function [acDimWL] = autocorrwidth(S, samp)
% [acDimWL] = autocorrwidth(slip, samp)
% returns the autocorrelation dimensions (width and length) for the slip distribution.
% For definition of autocorrelation width, see Mai and Beroza (2000)
% 
% NOTE:
% Adapted from EquiDIm2.m written by Mai, 2001, 2003
%
%
% 2014, Thingbaijam, K.K.S


% calculate cummulative slip in meters
Sz = sum(S, 2); % down-dip,
Sx = sum(S, 1); % along strike

% just to be sure
Sx = Sx(:)'; Sz = Sz(:)';

% pad a zero to avoid arbitary estimation of autocorrelation width
Sz = padarray(Sz,[0 1], 0);
Sx = padarray(Sx,[0 1], 0);

% calculate AutoCorrelation Functions (ACF) for along-strike, and 
% down-dip slip distributions, each ACF normalized to max. amp = 1
Cx = xcorr(Sx,Sx,'coeff');
Cz = xcorr(Sz,Sz,'coeff');
Nx = length(Cx);
Nz = length(Cz);

% integrate ACF's, and calculate Wacf by dividing through max. amp of ACF 
% (here it's simply unity, for a different choice of normalization, other
% divisors would occur)

L = samp(2)*trapz(1:1:Nx,Cx)/Cx((Nx+1)/2);
W = samp(1)*trapz(1:1:Nz,Cz)/Cz((Nz+1)/2);

acDimWL = [W, L];

