function [effmod] = effdimsrcmod2(fname, isfilename, useSPL, doplot, override)
% [effmod] = effdimsrcmod2(fname, isfilename, useSPL, doplot, override)
% calculate the effective source dimensions for input model provided as
% either a structure array or name of file for the SRCMOD MAT format. 
%
% INPUTS
% fname:        name of the file (including path and extensions) or structure array 
%               as defined in SRCMOD MAT format.
% isfilename:   (optional) a flag whether the input is name of MAT file or
%               structure array.
% useSPL:       =1, if Single Plane (SPL) format has to be used. 
%               The default value is 0.
%               
% doplot:       The options for plotting the figure. Default value is 0. 
%               0  - do not plot
%               1  - print to a file (myReport.ps). Usefull, when applying
%                    this function to number of slip models.   
%                    WARNING: this option closes all previous figures.    
%               2  - plot it on MATLAB fig file.  
%               3  - same as 2, but only the slip distribution. 
% override:     (optional) An integer array in form of [L, R, T, B], each
%               a value coresponds to number of subfualts to be trimmed 
%               on left, right, top and bottom. Providing this input argument 
%               overrides the algorithm by trimming the model according to the
%               provided number of subfualts to be trimmed (also see, mtrim.m)
%           
% OUTPUT
% effmod:   A structure array containing effective dimensions, and the trimmed model.
%
% NOTE 
% This function wraps the function - effdim, and intended to allow easy use of SRCMOD
% MAT file to calculate the effective source dimensions.
%
% CITE
% Thingbaijam K.K.S., and P.M. Mai. Evidence for truncated exponential probability
%       distribution of earthquake slip, Bulletin of the Seismological Society of America,
%       under review, submitted October, 2015.
% Mai, P. M., and G.C. Beroza. Source scaling properties from finite-fault-rupture 
%       models. Bulletin of the Seismological Society of America, 90(3), 604-615.
%
% AUTHOR
% 2014, Thingbaijam, K.K.S. (thingbaijam@gmail.com)
%
% SEE 
% readme.pdf for a disclaimer. 
% http://www.equake-rc.info/cers-software/ 

effmod = struct;

% parse the inputs
if nargin <5
    isoverride = 0;
else
    isoverride = 1;
end
if nargin<2
   isfilename = 1; useSPL = 0;  doplot = 0;
elseif nargin<3
   doplot = 0;
end

% check whether filename or stucture array is provided.
if isfilename
    dat     = load(fname);
    evTAG   = fieldnames(dat);
    M       = dat.(evTAG{1});
else
    M       = fname;
end

% check whether multi-segment fault or not
nseg = M.invSEGM;
if nseg==1
    useSPL = 1;
end

if useSPL==1     
    if doplot == 1
        close all; close all hidden; close force all;
        h = figure;
    elseif doplot == 2
        h = figure;
    end
    S       = M.slipSPL; 
    S       = abs(S);
    samp    = M.invDzDx; 
    htop    = M.srcZ2top;
    dip     = M.srcDipAn;
    
    % Do we have any edge artificat predefined?
    if isfield(M, 'edgefact')
        edgefact = M.edgefact;
    else
        edgefact = 0;
    end
    
    params.samp     = samp;
    params.htop     = htop;
    params.dip      = dip;
    params.edgefact = edgefact;
    
    if isoverride
        [dimWLeff, dimWL, tstats, is_surface] = effdim(S, params, doplot, override);
    else
        [dimWLeff, dimWL, tstats, is_surface] = effdim(S, params, doplot);
    end
    
    effmod.dimWLeff         = dimWLeff;
    effmod.dimWL            = dimWL;
    effmod.effArea          = dimWLeff(1)*dimWLeff(2);
    effmod.htop             = htop;
    effmod.samp             = samp;
    effmod.nseg             = nseg;
    effmod.tstats           = tstats;
    effmod.surfacerupture   = is_surface;
    
    if doplot == 1 
        titstr = sprintf('%s, Mw: %.2f, htop: %.1f', M.evTAG, M.srcMwMoS(1), htop);
        title(titstr);
        set(h, 'PaperOrientation', 'landscape');
        print(h, '-append', '-dpsc2', 'myReport.ps');
        close all;
        close all hidden;
    elseif doplot == 2
        titstr = sprintf('%s, Mw: %.2f, htop: %.1f', M.evTAG, M.srcMwMoS(1), htop);
        title(titstr);
    elseif doplot == 3
        title(M.evTAG, 'fontsize', 20);
    end
    
else
    totDimWL            = [0, 0];
    efftotDimWL         = [0, 0];
    effarea             = 0;
    effsurfacerupture   = 0;
    
    for i=1:nseg
        clear S h
                
        if doplot
            close all; close all hidden; close force all;
            h = figure;
        end
        
        S       = M.(strcat('seg',num2str(i),'SLIP')); %./100;  % I assume slip in cm
        htop    = M.(strcat('seg',num2str(i),'Z2top')); 
        dip     = M.(strcat('seg',num2str(i),'DipAn')); 
        samp    = M.invDzDx; 
        
        strsamp = strcat('seg',num2str(i),'invDzDx');
        tf      = isfield(M,strsamp);
        if tf==1
            samp = M.(strsamp); 
        end
                
        clear dimWLeff dimWL Seff
   
        if isfield(M, 'edgefact')
            edgefact = M.edgefact;
        else
            edgefact = 0;
        end
        
        params.samp     = samp;
        params.htop     = htop;
        params.dip      = dip;
        params.edgefact = edgefact;
    
            
        if isoverride
            [dimWLeff, dimWL, tstats, is_surface] = effdim(S, params, doplot, override);
        else
            [dimWLeff, dimWL, tstats, is_surface] = effdim(S, params, doplot);
        end
        
        effmod.(strcat('seg',num2str(i),'dimWLeff'))       = dimWLeff;
        effmod.(strcat('seg',num2str(i),'dimWL'))          = dimWL;
        effmod.(strcat('seg',num2str(i),'htop'))           = htop;
        effmod.(strcat('seg',num2str(i),'samp'))           = samp;
        effmod.(strcat('seg',num2str(i),'tstats'))         = tstats;
        effmod.(strcat('seg',num2str(i),'surfacerupture')) = is_surface;
        
        effarea             = effarea + dimWLeff(1)*dimWLeff(2);
        totDimWL(1)         = max(totDimWL(1),dimWL(1));
        totDimWL(2)         = totDimWL(2)+dimWL(2);
        efftotDimWL(1)      = max(efftotDimWL(1), dimWLeff(1));
        efftotDimWL(2)      = efftotDimWL(2) + dimWLeff(2);
        effsurfacerupture   = max(effsurfacerupture,is_surface);
        
        if doplot
            titstr = sprintf('%s, Mw: %.2f, seg: %d, htop: %.1f', M.evTAG, M.srcMwMoS(1), i, htop);
            title(titstr);
            set(h, 'PaperOrientation', 'landscape');
            print(h, '-append',  '-dpsc2', 'myReport.ps');
            close all;
            close all hidden;
        end
    end
    effmod.nseg             = nseg;
    effmod.dimWLeff         = efftotDimWL;
    effmod.dimWL            = totDimWL;
    effmod.effArea          = effarea;
    effmod.surfacerupture   = effsurfacerupture;
end





