
function effmod = effectivemodel(srcmod, SPL, override)
% effmod = effmodel(srcmod, SPL, override)
% 
% calculate the effective source dimensions for input model provided as
% structure array corresponding to SRCMOD MAT format. 
%  
% INPUTS
% srcmod:   A structure array as defined in SRCMOD MAT format.
% SPL:      (optional) a boolean flag (0 or 1), applicable for multi-segment models. If
%           SPL == 1, single-plane model is used even if the model has 
%           multiple segments. This flag is particularly usefull in adopting
%           single-plane model for down-dip segmented faults. 
%           Defualt value = 0.    
% override: (optional) An integer array in form of [L, R, T, B], each
%           a value coresponds to number of subfualts to be trimmed  
%           on left, right, top and bottom. Providing this input argument 
%           overrides the algorithm by trimming the model according to the
%           provided number of subfualts to be trimmed (also see, mtrim.m)
%
% OUTPUT
% effmod:   A structure array containing effective dimensions, and the trimmed model.
%
%
% NOTE
% This was originally written as effective_model.m.  This function is
% essentially the outermost wrapper to the function - effdimsrcmod2; and
% useful for process a number of models 
% 
%
% CITE
% Thingbaijam K.K.S., and P.M. Mai. Evidence for truncated exponential probability
%       distribution of earthquake slip, Bulletin of the Seismological Society of America,
%       under review, submitted October, 2015.
% Mai, P. M., and G.C. Beroza. Source scaling properties from finite-fault-rupture 
%       models. Bulletin of the Seismological Society of America, 90(3), 604-615.
%
% AUTHOR
% 2014, Thingbaijam, K.K.S. (thingbaijam@gmail.com)
%
% SEE 
% readme.pdf for a disclaimer. 
% http://www.equake-rc.info/cers-software/ 


if nargin<2; SPL = 0; end;


% caculate the effective slip distribution
if nargin<3
    [effmod] = compute_effsrcdim(srcmod, SPL);
else
    [effmod] = compute_effsrcdim(srcmod, SPL, override);
end

% include the model parameters
effmod.evTAG = srcmod.evTAG;
effmod.MwMoS = srcmod.srcMwMoS;

if ~effmod.segmented
    % for single-segmented faults 
    original_average_slip   = mean(srcmod.slipSPL(:));
    original_area           = effmod.dimWL(1)*effmod.dimWL(2);
    tS                      = mtrim(srcmod.slipSPL, effmod.tstats);
    
    if isfield(srcmod, 'riseSPL')
        tRise           = mtrim(srcmod.riseSPL, effmod.tstats);
        effmod.effrise  = tRise(:);
    end
    
    if isfield(srcmod, 'RISE')
        tRise           = mtrim(srcmod.RISE, effmod.tstats);
        effmod.effrise  = tRise(:);
    end
    
    effective_average_slip = mean(tS(:));
    effective_area         = effmod.dimWLeff(1)*effmod.dimWLeff(2);
    
    % rescale the slip distirbution
    compute_average_slip = (original_average_slip*original_area)/effective_area;
    change_factor        = compute_average_slip/effective_average_slip;
    tS                   = tS.*change_factor;   
    effmod.effslip       = tS(:)./100; % cm to m ?
    
    if isfield(srcmod, 'geoZ')
        effgeoZ = mtrim(srcmod.geoZ, effmod.tstats);
        effmod.effgeoZ = effgeoZ;
    end
    
    if isfield(srcmod, 'geoLAT')
        effgeoLAT = mtrim(srcmod.geoLAT, effmod.tstats);
        effmod.effgeoLAT = effgeoLAT;
    end
    
    if isfield(srcmod, 'geoLON')
        effgeoLON = mtrim(srcmod.geoLON, effmod.tstats);
        effmod.effgeoLON = effgeoLON;
    end
    
    [xx, yy] = size(tS);
    effmod.effslip2 = reshape(effmod.effslip,xx, yy); 
    
    if isfield(srcmod, 'geoZ')
        effmod.effHtop = min(effgeoZ(:));
    end
    
    if isfield(srcmod, 'srcDipAn')
        effmod.srcDipAn = srcmod.srcDipAn;
    end
    
    if isfield(srcmod, 'srcAStke')
        effmod.srcAStke = srcmod.srcAStke;
    end
    
else
    % for multi-segmented faults
    tS = []; tR = []; rise_found = 0;
    
    % TO DO: some optimization can be applied here so as to pre-allocation 
    % of memory for some variables  
    for i=1: effmod.nseg
        clear S subfslip original_average_slip original_area
        clear effective_average_slip effective_area compute_average_slip
        clear change_factor dimWL effDimWL tGeoZ
        
        subfslip                = srcmod.(strcat('seg', int2str(i), 'SLIP'))./100;
        original_average_slip   = mean(subfslip(:));
        dimWL                   = effmod.(strcat('seg', int2str(i), 'dimWL'));  
        original_area           = dimWL(1)*dimWL(2);
        
        S = mtrim(subfslip, effmod.(strcat('seg', int2str(i), 'tstats')));
        
        risestr = strcat('seg', int2str(i), 'RISE');
        if isfield(srcmod, risestr)
            tRise       = mtrim(srcmod.(risestr), effmod.(strcat('seg', int2str(i), 'tstats')));
            rise_found  = 1;
        end
        
        geZstr = strcat('seg', int2str(i), 'geoZ');
        if isfield(srcmod, geZstr)
            tGeoZ           = mtrim(srcmod.(geZstr), effmod.(strcat('seg', int2str(i), 'tstats')));
            segeffHtop(i)   = min(tGeoZ(:));
            effmod.(strcat('seg', int2str(i),'effGeoZ')) = tGeoZ;
        end 
        
        
        effective_average_slip  = mean(S(:));
        effDimWL                = effmod.(strcat('seg', int2str(i), 'dimWLeff'));  
        effective_area          = effDimWL(1)*effDimWL(2);
        % scale the slip distribution 
        compute_average_slip    = (original_average_slip*original_area)/effective_area;
        change_factor           = compute_average_slip/effective_average_slip;
        S                       = S.*change_factor;  
        tS = [tS, S(:)'];
        
        if rise_found
            tR = [tR, tRise(:)'];
        end
        effmod.(strcat('seg', int2str(i), 'slipeff')) = S;
    end
    effmod.effslip = tS;
    effmod.effHtop = min(segeffHtop(:));
    if rise_found
        effmod.effrise = tR;
    end
end


function  effmod = compute_effsrcdim(M, SPL, override)
% effmod = compute_effsrcdim(M, SPL,override)
% 
% INPUTS
% same as for effectivemodel()
% 
% OUTPUT
% same as for effectivemodel()
% 
% Note:
% Second wrapper to effdimsrcmod2()
% 
% 2014, Thingbaijam, K.K.S. 


% check on whether to use single-plane model or not, irrespective of user's
% input. This is becuase of the data constraints.

if SPL == 0
    if M.invSEGM == 1
        useSPL = 1;
    else
        useSPL = 1;     
        % This checks whether fault segmentation is feasible for the calculations.
        % That is, whether fault-segmenation is along the strike or not. 
        if isfield(M, 'segmented')         
            if strcmpi(M.segmented,'strike') 
                useSPL = 0;
            end
        end
    end
else
    useSPL = 1;
end

segmented = ~useSPL;

if nargin <3
    [effmod] = effdimsrcmod2(M, 0, useSPL, 0);
else
    [effmod] = effdimsrcmod2(M, 0, useSPL, 0,override);
end

effmod.segmented = segmented;

 


